% Matlab stuff for the new ISS lecture on spectrum 
% Honza Tue Sep 29 10:39:39 CEST 2015

% generating "continuous" cos
n= 1:500; 
x = cos (2 * pi * n / 250); 
plot (x);axis off; print -dpng fig/cosonly.png 

% a faster one 
x = cos (4 * pi * n / 250); 
plot (x);axis off; print -dpng fig/cosonly2.png 

%%% DEMO 1 - cos 440 Hz %%%
% 440 Hz on Fs = 8000 - 1 second 
n = 0:7999; Fs = 8000; 
x = cos (2 * pi * 440 / Fs * n); 
plot (n,x);   % <--- zoom it and try to compute the period, convert fo f_skut
% print -dpng fig/440Hz.png
soundsc (x,8000); 

% phases ... showing still with 440 Hz ... 50 samples
n = 0:50; ii=1; 
for phi = (0:pi/4:3/4*pi)
  x = cos (2 * pi * 440 / Fs * n + phi); 
  subplot (4,1,ii); plot(n,x); text(30,0.5,['\phi=' num2str(phi)],'FontSize',30); 
  ii = ii+1; 
end 
print -dpng fig/phase1.png 
ii=1; 
for phi = (pi:pi/4:7/4*pi)
  x = cos (2 * pi * 440 / Fs * n + phi); 
  subplot (4,1,ii); plot(n,x); text(30,0.5,['\phi=' num2str(phi)],'FontSize',30); 
  ii = ii+1; 
end 
print -dpng fig/phase2.png 

%%% DEMO2 - "Ovcaci ctveraci" 
% durations and heights
H = [1 5 8 1 5 8 5 5 3 5 6 3 5 5 3 5 6 3 5 3 1]; 
D = [2 2 4 2 2 4 1 1 1 1 2 2 1 1 1 1 2 2 2 2 4]; 
base = 440; Fs = 8000; halftone = 2^(1/12); basedur=1000; 
xx = []; % very terrible, but what ... 
for ii=1:length(H), 
  x = cos (2 * pi * base / Fs * halftone^(H(ii)-1) * (1:(basedur * D(ii)))); 
  xx = [xx x]; 
end
%wavwrite (xx,8000,16,'sig/ovcaci.wav'); 
soundsc(xx,8000); 

% -------------- analysis -------------------
% DC signal first, the analysis one will be always drawn as red 
n = 0:99; 
a = ones(size(n)); plot(n,x,'r'); title('a[n]'); grid; print -dpng fig/anal_dc.png 

x =  5 * ones(size(n)); 
subplot(311); plot(n,x); title('x[n]'); grid;
subplot(312); plot(n,a,'r'); title('a[n]'); grid;
xa = x.*a; 
subplot(313); plot(n,xa); title('x[n] a[n]'); grid;
coef = sum (xa); text(75,mean(xa),['\Sigma = ' num2str(coef)],'FontSize',30); 
print -dpng fig/anal_dc_dc.png 

x =  2.5 * ones(size(n)) + 2.5*cos(2*pi*n/25); 
subplot(311); plot(n,x); title('x[n]'); grid;
subplot(312); plot(n,a,'r'); title('a[n]'); grid;
xa = x.*a; 
subplot(313); plot(n,xa); title('x[n] a[n]'); grid;
coef = sum (xa); text(75,mean(xa),['\Sigma = ' num2str(coef)],'FontSize',30); 
print -dpng fig/anal_dc_cos.png 

x =   2.5*cos(2*pi*n/25); 
subplot(311); plot(n,x); title('x[n]'); grid;
subplot(312); plot(n,a,'r'); title('a[n]'); grid;
xa = x.*a; 
subplot(313); plot(n,xa); title('x[n] a[n]'); grid;
coef = sum (xa); text(75,mean(xa),['\Sigma = ' num2str(coef)],'FontSize',30); 
print -dpng fig/anal_dc_coscentered.png 

x =  -2.5 * ones(size(n)) + 0.5*cos(2*pi*n/25); 
subplot(311); plot(n,x); title('x[n]'); grid;
subplot(312); plot(n,a,'r'); title('a[n]'); grid;
xa = x.*a; 
subplot(313); plot(n,xa); title('x[n] a[n]'); grid;
coef = sum (xa); text(75,mean(xa),['\Sigma = ' num2str(coef)],'FontSize',30); 
print -dpng fig/anal_dc_minuscos.png 

%----  now analyzing function is a cos ...
close all; 
a = cos(3* 2*pi*n/100);
plot(n,x,'r'); title('a[n]'); grid; print -dpng fig/anal_cos.png 

x =  5 * ones(size(n)); 
subplot(311); plot(n,x); title('x[n]'); grid;
subplot(312); plot(n,a,'r'); title('a[n]'); grid;
xa = x.*a; 
subplot(313); plot(n,xa); title('x[n] a[n]'); grid;
coef = sum (xa); text(75,mean(xa),['\Sigma = ' num2str(coef)],'FontSize',30); 
print -dpng fig/anal_cos_dc.png 

x =  cos(3* 2*pi*n/100);
subplot(311); plot(n,x); title('x[n]'); grid;
subplot(312); plot(n,a,'r'); title('a[n]'); grid;
xa = x.*a; 
subplot(313); plot(n,xa); title('x[n] a[n]'); grid;
coef = sum (xa); text(75,mean(xa),['\Sigma = ' num2str(coef)],'FontSize',30); 
print -dpng fig/anal_cos_cos.png 

% a bit noised one 
x =  cos(3* 2*pi*n/100)+0.1 * randn(size(n));
subplot(311); plot(n,x); title('x[n]'); grid;
subplot(312); plot(n,a,'r'); title('a[n]'); grid;
xa = x.*a; 
subplot(313); plot(n,xa); title('x[n] a[n]'); grid;
coef = sum (xa); text(75,mean(xa),['\Sigma = ' num2str(coef)],'FontSize',30); 
print -dpng fig/anal_cos_cosnoise.png 

% incl D.C. shift 
x =  5+ cos(3* 2*pi*n/100); 
subplot(311); plot(n,x); title('x[n]'); grid;
subplot(312); plot(n,a,'r'); title('a[n]'); grid;
xa = x.*a; 
subplot(313); plot(n,xa); title('x[n] a[n]'); grid;
coef = sum (xa); text(75,mean(xa),['\Sigma = ' num2str(coef)],'FontSize',30); 
print -dpng fig/anal_cos_cosdc.png 

% negative 
x =  -cos(3* 2*pi*n/100);  
subplot(311); plot(n,x); title('x[n]'); grid;
subplot(312); plot(n,a,'r'); title('a[n]'); grid;
xa = x.*a; 
subplot(313); plot(n,xa); title('x[n] a[n]'); grid;
coef = sum (xa); text(75,mean(xa),['\Sigma = ' num2str(coef)],'FontSize',30); 
print -dpng fig/anal_cos_cosminus.png 

% -------- battery of cosines ! ------------------------
% first needs to generate some nice signal ... 
% will try to color it by a bit of speech !
ff = fopen('sig/test.l16','r'); s = fread (ff,[1 inf],'short'); fclose(ff); 
plot(s); x = s(1500:1699); a = lpc(x,10); H = abs(freqz(1,a,51));
% now generate the signal ... 
n = 0:99;  
x = zeros(size(n)); 
for ii=1:51,
  % phase should be random ... 
  phi = rand(1,1) * 2 * pi; 
  xx = H(ii) * cos(2 * pi * (ii - 1) / 100 * n + phi); %plot(xx); pause 
  x = x + xx; 
end
%remove mean and normalize ... 
x = x - mean(x); x = x / max(abs(x)); 
% clone it 80x and save - 1 second of a signal ... 
s = repmat(x,1,80); 
%wavwrite(s, 8000, 16, 'sig/signal.wav'); 

%%%%%%%%%%%%% DEMO 3 - analysis of signal by battery of coses ... 
% ok, now using the one that was generated ... select 100 samples 
s = wavread('sig/signal.wav'); plot (s); x = s(1:100); plot(x); 
% needs to generate some basis - will be in rows.  
A = zeros(51,100); 
n = 0:99;  
for ii=1:51, 
  fnorm = (ii - 1) / 100; 
  a = cos(2 * pi * fnorm * n); plot(a); soundsc(a); pause; 
  A(ii,:) = a; 
end
% now compute the projection 
c = A * x; 
% show them 
plot(c); print -dpng fig/c_bad.png  % hm hm 
% show them again 
plot(abs(c)); print -dpng fig/c_bad_abs.png  % not terribly nice ... 

%%% trying to synthesize the thing back. 
xs = c' * A; plot (n,x,n,xs); % hm hm 
% trying to re-normalize it ... 
xs = xs / max(abs(xs)); plot (n,x,n,xs); % hm hm. 
% clone it and listen to it ... 
xxs = repmat(xs,1,80); 
%wavwrite(xxs, 8000, 16, 'sig/signal_syn_bad.wav');  
soundsc (xxs); % well ... 

% ----------- faze a siny ... ------------------
% zopakovat anal_cos_cos a anal_cos_minux  ...
n = 0:99; 
a = cos(3* 2*pi*n/100);
x =  cos(3* 2*pi*n/100 + pi/2);  
subplot(311); plot(n,x); title('x[n]'); grid;
subplot(312); plot(n,a,'r'); title('a[n]'); grid;
xa = x.*a; 
subplot(313); plot(n,xa); title('x[n] a[n]'); grid;
coef = sum (xa); text(75,mean(xa),['\Sigma = ' num2str(coef)],'FontSize',30); 
print -dpng fig/anal_cos_sin.png 

%%%%%%% DEMO 4 cos a siny ... %%%%%%%%%%%%%
n = 0:99; 
a = cos(3* 2*pi*n/100);
b = sin(3* 2*pi*n/100);
for phi = 0:2*pi/10:2*pi, 
  x =  cos(3* 2*pi*n/100 + phi);  
  subplot(321); plot(n,x); title('x[n]'); grid;
  %cos
  subplot(323); plot(n,a,'r'); title('a[n]'); grid;
  xa = x.*a; 
  subplot(325); plot(n,xa); title('x[n] a[n]'); grid;
  c = sum (xa); text(75,mean(xa),['c = ' num2str(c)],'FontSize',15); 
  % sin
  subplot(324); plot(n,b,'r'); title('b[n]'); grid;
  xb = x.*b; 
  subplot(326); plot(n,xb); title('x[n] b[n]'); grid;
  d = sum (xb); %text(75,mean(xb),['\Sigma = ' num2str(d)],'FontSize',30); 
  e = sqrt(c ^2 + d^2); text(10,mean(xb),['d = ' num2str(d) ' (c^2 + d^2)^{-1/2}=' num2str(e)],'FontSize',15); 
  pause; 
end

%%%%%%% DEMO 4a dtto with showing the output as a complex number - no. 

%%%%%%% DEMO 5  analyza baterii sinu a cosinu ... 
s = wavread('sig/signal.wav'); plot (s); x = s(1:100); plot(x); 
% needs to generate some basis - will be in rows.  
A = zeros(51,100); 
B = zeros(51,100); 
n = 0:99;  
for ii=1:51, 
  fnorm = (ii - 1) / 100; 
  a = cos(2 * pi * fnorm * n); 
  b = sin(2 * pi * fnorm * n); plot(n,a,n,b);  pause; 
  A(ii,:) = a;   B(ii,:) = b;
end
% now compute the projection 
c = A * x; d = B * x;
X = c - j*d; 
plot (abs(X)); 

% vygenerovat a zahrat 
xs = c' * A + d' * B; plot (n,x,n,xs); %   still a size problem ... 
% trying to re-normalize it ... 
xs = xs / max(abs(xs)); plot (n,x,n,xs); %  nice ! 
% clone it and listen to it ... 
xxs = repmat(xs,1,80); 
%wavwrite(xxs, 8000, 16, 'sig/signal_syn_good.wav');  
soundsc (xxs); 

%%%%%%%%%%% DEMO 6 complex exponentials %%%%%%%%%%%%%%%
N = 100; n = 0:(N-1);
f = 1/N; 
for k=0:20,
  ce = exp(-j *2 *pi * f * k * n); 
  subplot (311); plot3(n,real(ce),imag(ce)); 
  subplot (312); plot(n,real(ce));
  subplot (313); plot(n,imag(ce)); pause
end

%%%%%%%%%%%%% DEMO 7 analysis with DF %%%%%%%%%%%%%%
s = wavread('sig/signal.wav'); plot (s); x = s(1:100); plot(x); 
X = fft(x); 
% visualizing just 1/2 of points ... 
Xshow = abs(X(1:51)); 
plot (Xshow); 
% and a nice frequency axis 
k = 0:50; 
f = k / 100 * 8000; 
plot(f,Xshow); 

%%%%%%%%%%%%% DEMO 8 synthesis  %%%%%%%%%%%%%%
xs = ifft(X);  plot (n,x,n,xs); % no need of re-norm !  nice !
% clone it and listen to it ... 
xxs = repmat(xs,80,1); % grrr, return of ifft is column ...  
%wavwrite(xxs, 8000, 16, 'sig/signal_syn_fft.wav');  
soundsc (xxs); % veeery nice. 













